//---------------------------------------------------------------------------


#pragma hdrstop

#include "CallbackQueue.h"
#include "Callback.h"
#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

#include "fifo.h"
#include "Callback.h"

namespace
{
	Fifo<Callback, 512> fifo;
	Mutex mutex;
}

CallbackQueue::CallbackQueue()
{

}

int CallbackQueue::GetCallback(Callback& cb)
{
	ScopedLock<Mutex> lock(mutex);
	Callback *tmpcb = (Callback*)fifo.getReadable();
	if (tmpcb == NULL)
	{
		return 1;
	}
	cb = *tmpcb;
	fifo.pop();
	return 0;
}

void CallbackQueue::ChangeCallState(Callback::ua_state_e state, AnsiString caller)
{
	ScopedLock<Mutex> lock(mutex);
	Callback *cb = (Callback*)fifo.getWriteable();
	if (!cb)
		return;
	cb->type = Callback::CALL_STATE;
	cb->state = state;
	cb->caller = caller;
	fifo.push();
}

void CallbackQueue::ChangeRegState(Callback::reg_state_e state, int acc_id)
{
	ScopedLock<Mutex> lock(mutex);
	Callback *cb = (Callback*)fifo.getWriteable();
	if (!cb)
		return;
	cb->type = Callback::REG_STATE;
	cb->reg_state = state;
	cb->accountId = acc_id;
	fifo.push();
}

void CallbackQueue::ChangeAppState(Callback::app_state_e state)
{
	ScopedLock<Mutex> lock(mutex);
	Callback *cb = (Callback*)fifo.getWriteable();
	if (!cb)
		return;
	cb->type = Callback::APP_STATE;
	cb->app_state = state;
	fifo.push();
}


